// Make a Loonie shaped coaster with an internal lithophane
// and with small ridges to prevent flat-bottomed glasses
// from sticking to the coaster 

$fn = 150;

// Image should have some completely black pixels
// that will be printed at full height
// so that layer on top can attach to them.
image="pic-cannabis-200x200.png";
im_x = 200;
im_y = 200;
im_resize = 1.01;

n_sides = 11;
object_hgt = 4.5;
outer_wid = 80;
wall_wid = 2;
bottom_layer = 0.4;
top_layer = 0.5;
img_hgt = 2;
inner_wid = outer_wid - (2*wall_wid);
cavity_hgt = object_hgt - img_hgt - bottom_layer - top_layer;
ridges_hgt = 0.8;
ridge_wid = 0.8;

bevel_1 = (outer_wid+(2*wall_wid)+2)/outer_wid;
bevel_2 = (bevel_1 - 1) * (img_hgt/object_hgt) + 1; 
bevel_3 = (bevel_1 - 1) * (cavity_hgt/object_hgt) + 1; 

cavity_wid = bevel_3 * inner_wid;
cavity_wid = ((bevel_1 - 1) * ((object_hgt-cavity_hgt)/object_hgt) + 1) * inner_wid;

union(){
  difference(){
    union(){
      difference(){ // Put % in front to see image in preview (also do it below)
        linear_extrude(height = object_hgt, scale=bevel_1) reuleaux_polygon(n_sides, outer_wid);
        translate([0, 0, bottom_layer]) linear_extrude(height = img_hgt, scale=bevel_2) reuleaux_polygon(n_sides, inner_wid);
      }
      intersection(){
        translate([0, 0, bottom_layer]) linear_extrude(height = img_hgt, scale=bevel_2) reuleaux_polygon(n_sides, inner_wid);
        translate([0, 0, img_hgt+bottom_layer]) scale([im_resize*inner_wid/im_x, im_resize*inner_wid/im_y, .01*img_hgt])
            surface(file=image, invert=true, center=true);
      } 
    }
    translate ([0,0,object_hgt-cavity_hgt]) linear_extrude(height = cavity_hgt, scale=bevel_3) reuleaux_polygon(n_sides, cavity_wid);
  }
  // Raised ridges so that the glass doesn't stick to it if it's wet
  starbase = inner_wid/2;
  translate ([0,0,object_hgt-cavity_hgt]) // Put % in front to see image in preview (also do it above)
  union () {
    color ("blue") linear_extrude(height = 0.6, center = false, convexity = 10, twist = 0)
    rotate(90)
    difference() {
      Star(11,starbase*(75/75),starbase*(75/75)*0.4);
      Star(11,starbase*(60/75),starbase*(60/75)*0.4);
    }
    color ("red") linear_extrude(height = 0.72, center = false, convexity = 10, twist = 0)
    rotate(90)
    difference() {
      Star(11,starbase*(72/75),starbase*(72/75)*0.4);
      Star(11,starbase*(63/75),starbase*(63/75)*0.4);
    }
  }
}



//  Produce a Reuleaux polygon

//  Reuleaux polygons are examples of curves of constant width.  The wheel is
//  the simplest such curve.  Reuleaux polygons are the next simplest, albeit
//  they suffer from tangent continuity.  Curves of constant width can be used,
//  for example, as wheels and hole covers (which cannot be oriented such that
//  they can fall through the opening they are covering).

//  Dan Newman, dan newman @ mtbaldy us
//  16 June 2012 
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License,
//  LGPL version 2.1, or (at your option) any later version of the GPL.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 

//  Produce a Reuleaux polygon of maximal width w and n sides
//  Note that n must be odd-valued.  Reason for that is that
//  the construction of a Reuleaux polygon requires each vertex V
//  to have two opposite vertices which are equidistant from V.

module reuleaux_polygon(n = 3, w = 10)
{
     // Angular separation between vertices of the regular n-gon
     angle = 360 / n;

     // If the n-gon's edge length is 1 then the distance from the
     // center to a vertex is
     r = 1 / (2 * sin(angle/2));

     // Place the first vertex, V0, at [0, r]
     V0 = [0, r];

     // We now wish to know the distance from V0 to either of the
     // two opposite vertices

     // Compute the angle from V0 to one of the opposite vertices  
     theta = angle * floor(n / 2);

     // Compute the coordinates of this opposite vertex
     Vopposite = [ V0 * [cos(theta), -sin(theta)],
                   V0 * [sin(theta),  cos(theta)] ];
 
     // What's the distance between these two vertices?
     d = sqrt( pow(V0[0] - Vopposite[0], 2) + pow(V0[1] - Vopposite[1], 2) );

     // Finally, the scaling to rescale d to be w
     s = w / d;
     intersection_for(i = [0 : n-1])
     {
          rotate([0, 0, i * angle])
              translate([0, s * r, 0])
                  circle(r = s * d, center=true);
     }
}

// Produce a star
// points = number of points (minimum 3)
// outer  = radius to outer points
// inner  = radius to inner points
module Star(points, outer, inner) {
	
	// polar to cartesian: radius/angle to x/y
	function x(r, a) = r * cos(a);
	function y(r, a) = r * sin(a);
	
	// angular width of each pie slice of the star
	increment = 360/points;
	
	union() {
		for (p = [0 : points-1]) {
			
			// outer is outer point p
			// inner is inner point following p
			// next is next outer point following p

			assign(	x_outer = x(outer, increment * p),
					y_outer = y(outer, increment * p),
					x_inner = x(inner, (increment * p) + (increment/2)),
					y_inner = y(inner, (increment * p) + (increment/2)),
					x_next  = x(outer, increment * (p+1)),
					y_next  = y(outer, increment * (p+1))) {
				polygon(points = [[x_outer, y_outer], [x_inner, y_inner], [x_next, y_next], [0, 0]], paths  = [[0, 1, 2, 3]]);
			}
		}
	}
}

